module MatchMate
  class Address
    def self.languages
      @languages ||= MatchMate.config.languages || DEFAULTS[:languages]
    end

    def initialize(input_object_or_string)
      # Accepts any address object that responds to a to_s method
      @input_string = input_object_or_string.to_s
    end

    # Basic Components for comparsion
    def road
      expand components[:road]
    end

    def unit
      extract_integers components[:unit]
    end

    def postcode
      limit_to components[:postcode], MatchMate.config.postcode_limit
    end

    def city
      components[:city]
    end

    def state
      components[:state]
    end

    def house_number
      components[:house_number]
    end

    def ==(other_address)
      AddressMatcher.new(self, other_address).match?
    end

    alias eql? ==
    alias === ==

    # Normalized Compositites

    def city_and_state
      [city, state].compact.join(', ')
    end

    def street_address
      [house_number, road.last].compact.join(' ')
    end

    def street_adress_with_unit
      [street_address, unit].compact.join(' ')
    end

    def to_s
      [street_adress_with_unit, city_and_state, zip_code].compact.join(', ')
    end

    private

    def extract_integers(string, limit = nil)
      return if string.blank?

      value = string.gsub(/\D/, '')
      limit ? value.first(limit) : value
    end

    def components
      {}.tap do |components|
        parsed_object.each do |component|
          components[component[:label]] = component[:value]
        end
      end
    end

    def parsed_object
      @parsed_object ||= Postal::Parser.parse_address @input_string, languages: self.class.languages
    end

    def limit_to(string, limit)
      return string if string.blank? || limit.blank?

      string.first(limit)
    end

    def expand(string)
      return if string.blank?

      Postal::Expand.expand_address string, languages: self.class.languages
    end
  end
end
