module MatchMate
  class AddressMatcher
    def self.attributes
      @attributes ||= (MatchMate.config.address_weights || {}).deep_merge DEFAULTS[:address_weights]
    end

    def self.threshold
      @threshold ||= MatchMate.config.address_threshold || DEFAULTS[:address_threshold]
    end

    def self.total_weight
      @total_weight ||= attributes.values.sum { |config| config[:weight] }
    end

    def initialize(address, other_address)
      @address = address.is_a?(MatchMate::Address) ? address : Address.new(address)
      @other_address = other_address.is_a?(MatchMate::Address) ? other_address : Address.new(other_address)
    end

    def score
      @score ||= calculate_score.round
    end

    def match?
      score >= self.class.threshold
    end

    private

    def calculate_score
      self.class.attributes.keys.sum do |attribute|
        weighted_score_for attribute
      end
    end

    def weighted_score_for(attribute)
      value = score_for(attribute)
      config = self.class.attributes[attribute]
      return 0 if value < config[:threshold]

      value * adjusted_weight_for(config[:weight])
    end

    def adjusted_weight_for(weight)
      ((self.class.total_weight.to_f / 100) * weight) / 100
    end

    def score_for(attribute)
      FuzzyWuzzy.new(@address.send(attribute), @other_address.send(attribute)).token_set_ratio
    end
  end
end
