PyCall.init MatchMate.config.python_path || `which python`.chomp

class FuzzyWuzzy
  def initialize(value, other_value)
    @values = [value].flatten.map(&:to_s)
    @other_values = [other_value].flatten.map(&:to_s)
    PyCall.exec import
  end

  # Compares the entire string similarity, ignores word order and duplicated words
  def token_set_ratio
    run_command :token_set_ratio
  end

  # Compares the entire string similarity, ignores word order
  def token_sort_ratio
    run_command :token_sort_ratio
  end

  # Compares partial string similarity
  def partial_ratio
    run_command :partial_ratio
  end

  # Compares the entire string similarity, in order.
  def ratio
    run_command :ratio
  end

  private

  def run_command(method)
    scores = []
    combinations.each do |value, other_value|
      score = score_for method, value, other_value
      scores << score
      break if score == 100
    end
    scores.max
  end

  def score_for(method, value, other_value)
    value = normalize(value)
    other_value = normalize(other_value)
    if value == other_value
      100
    else
      PyCall.eval command(method, value, other_value)
    end
  end

  def combinations
    @values.product @other_values
  end

  def import
    <<~PYTHON
      from fuzzywuzzy import fuzz
    PYTHON
  end

  def normalize(value)
    value.strip.squeeze(' ').downcase
  end

  def command(method, value, other_value)
    <<~PYTHON
      fuzz.#{method}("#{value}", "#{other_value}")
    PYTHON
  end
end
